//////////////////////////////////////////////////////////////////////
//                              ExecModify                       
// Copyright 2014 Malcolm Stagg
// Modified Version 1.0: 06/05/2014
//
//////////////////////////////////////////////////////////////////////

/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this program.  If not, see
<http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

#include "inc/IProcessorOpcode.h"
#include "inc/IExecutableParser.h"
#include "inc/ExecParseUtil.h"

const char * SectionTypeString[22] = {
	"None",
	"Unknown",
	"ProgramBits",
	"SymbolTable",
	"StringTable",
	"Rela",
	"Hash",
	"Dynamic",
	"Note",
	"NoBits",
	"Reloc",
	"ShLib",
	"DynSym",
	"LoProc",
	"HiProc",
	"LoUser",
	"HiUser",
	"Text",
	"Bss",
	"Rdata",
	"Data",
	"Idata"
};

const char * SymbolTypeString[8] = {
	"None",
	"Unknown",
	"Object",
	"Function",
	"Section",
	"File",
	"LoProc",
	"HiProc"
};

const char * SymbolBindingString[7] = {
	"None",
	"Unknown",
	"Local",
	"Global",
	"Weak",
	"LoProc",
	"HiProc"
};

Endianess GetHostByteOrder() {
	uint32_t * intdata;
	uint8_t bytedata[4] = { 0x12, 0x34, 0x56, 0x78 };
	intdata = (uint32_t *)bytedata;

	if (*intdata == 0x12345678) {
		return big_endian;
	}
	else if (*intdata == 0x78563412) {
		return little_endian;
	}

	return unknown_endian;
}

void ConvertEndian64(uint64_t &number, Endianess from_endian, Endianess to_endian) {
	if (from_endian != to_endian)
	{
		number =
			(number << 56) & 0xFF00000000000000ULL |
			(number << 40) & 0x00FF000000000000ULL |
			(number << 24) & 0x0000FF0000000000ULL |
			(number << 8) & 0x000000FF00000000ULL |
			(number >> 8) & 0x00000000FF000000ULL |
			(number >> 24) & 0x0000000000FF0000ULL |
			(number >> 40) & 0x000000000000FF00ULL |
			(number >> 56) & 0x00000000000000FFULL;
	}
}

void ConvertEndian32(uint32_t &number, Endianess from_endian, Endianess to_endian) {
	if (from_endian != to_endian)
	{
		number = (number << 24) & 0xFF000000 | (number << 8) & 0x00FF0000 | (number >> 8) & 0x0000FF00 | (number >> 24) & 0x000000FF;
	}
}

void ConvertEndian16(uint16_t &number, Endianess from_endian, Endianess to_endian) {
	if (from_endian != to_endian)
	{
		number = (number << 8) & 0xFF00 | (number >> 8) & 0x00FF;
	}
}

int64_t GetNumber(const char * szOptionVal)
{
	bool fNegative = false;
	int64_t retval = 0LL;
	int64_t digit = 0LL;
	if (szOptionVal[0] == '-')
	{
		fNegative = true;
		szOptionVal++;
	}

	if (szOptionVal[0] == '0' && szOptionVal[1] == 'x')
	{
		szOptionVal+=2;
		// hex
		while (szOptionVal[0])
		{
			if (szOptionVal[0] >= 'A' && szOptionVal[0] <= 'F')
			{
				digit = szOptionVal[0] - 'A' + 10;
			}
			else if (szOptionVal[0] >= 'a' && szOptionVal[0] <= 'f')
			{
				digit = szOptionVal[0] - 'a' + 10;
			}
			else if (szOptionVal[0] >= '0' && szOptionVal[0] <= '9')
			{
				digit = szOptionVal[0] - '0';
			}
			else
			{
				break;
			}

			retval *= 16LL;
			retval += digit;
			szOptionVal++;
		}
	}
	else
	{
		// assume decimal
		while (szOptionVal[0])
		{
			if (szOptionVal[0] >= '0' && szOptionVal[0] <= '9')
			{
				digit = szOptionVal[0] - '0';
			}
			else
			{
				break;
			}

			retval *= 10LL;
			retval += digit;
			szOptionVal++;
		}
	}

	if (fNegative)
	{
		retval = -retval;
	}

	return retval;
}
