//////////////////////////////////////////////////////////////////////
//                              ExecModify                       
// Copyright 2014 Malcolm Stagg
// Modified Version 1.0: 06/05/2014
//
//////////////////////////////////////////////////////////////////////

/*
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as 
published by the Free Software Foundation, either version 3 of the 
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this program.  If not, see 
<http://www.gnu.org/licenses/>.
*/

#ifndef __I_EXECUTABLE_PARSER_H__
#define __I_EXECUTABLE_PARSER_H__

#pragma once

// add to these structures to support ELF, PE, maybe more types in the future

enum SectionType
{
	SectionType_None = 0,
	SectionType_Unknown = 1,
	SectionType_ProgramBits = 2,
	SectionType_SymbolTable = 3,
	SectionType_StringTable = 4,
	SectionType_Rela = 5,
	SectionType_Hash = 6,
	SectionType_Dynamic = 7,
	SectionType_Note = 8,
	SectionType_NoBits = 9,
	SectionType_Reloc = 10,
	SectionType_ShLib = 11,
	SectionType_DynSym = 12,
	SectionType_LoProc = 13,
	SectionType_HiProc = 14,
	SectionType_LoUser = 15,
	SectionType_HiUser = 16,
	SectionType_Text = 17,
	SectionType_Bss = 18,
	SectionType_Rdata = 19,
	SectionType_Data = 20,
	SectionType_Idata = 21
};

extern const char * SectionTypeString[22];

typedef struct
{
	char szName[256];
	SectionType type;
	uint64_t uiOffset;
	uint64_t uiSize;
} SectionInformation;

enum SymbolType
{
	SymbolType_None = 0,
	SymbolType_Unknown = 1,
	SymbolType_Object = 2,
	SymbolType_Function = 3,
	SymbolType_Section = 4,
	SymbolType_File = 5,
	SymbolType_LoProc = 6,
	SymbolType_HiProc = 7
};

extern const char * SymbolTypeString[8];

enum SymbolBinding
{
	SymbolBinding_None = 0,
	SymbolBinding_Unknown = 1,
	SymbolBinding_Local = 2,
	SymbolBinding_Global = 3,
	SymbolBinding_Weak = 4,
	SymbolBinding_LoProc = 5,
	SymbolBinding_HiProc = 6
};

extern const char * SymbolBindingString[7];

typedef struct
{
	char szName[256];
	SymbolType type;
	SymbolBinding binding;
	uint64_t uiVirtualAddress;
	uint64_t uiSize;
} SymbolInformation;

class IExecutableParser
{
public:
	virtual bool LoadFile(const char * szFileName) = 0;
	virtual bool CloseFile() = 0;
	virtual bool GetFirstSection() = 0;
	virtual bool GetNextSection() = 0;
	virtual bool FindSection(char * szSectionName, SectionType type) = 0;
	virtual bool PrintSectionInformation() = 0;
	virtual bool GetSectionInformation(SectionInformation &sectionInfo) = 0;
	virtual bool GetFirstSymbol() = 0;
	virtual bool GetNextSymbol() = 0;
	virtual bool FindSymbol(char * szSymbolName, SymbolType type) = 0;
	virtual bool PrintSymbolInformation() = 0;
	virtual bool GetSymbolInformation(SymbolInformation &symbolInfo) = 0;
	virtual bool GetFunctionVirtualAddress(char * szSymtab, char * szFunctionName, uint64_t &uiVirtualAddress, uint64_t &uiSize) = 0;
	virtual bool GetEntryPointVirtualAddress(uint64_t &uiVirtualAddress) = 0;
	virtual bool VirtualMemoryToFileOffset(uint64_t uiVirtualAddress, uint64_t &uiOffset) = 0;
	virtual bool FileOffsetToVirtualMemory(uint64_t uiOffset, uint64_t &uiVirtualAddress) = 0;
	virtual bool GetProcessorOpcodes(IProcessorOpcode * &pProcOpcode) = 0;
	virtual bool OverwriteData(uint64_t uiOffset, char * cData, int cbData) = 0;
};

#endif